<?php

namespace Drupal\go_back\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Cache\Cache;

/**
 * Provides a Go Back Block with his display.
 *
 * @Block(
 *   id = "go_back",
 *   admin_label = @Translation("Go Back Block"),
 *   category = @Translation("Go Back Block"),
 * )
 */
class GoBack extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity manager type service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity manager service.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplay;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity.manager'),
      $container->get('entity_display.repository')
    );
  }

  /**
   * This function construct a block.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity manager service.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entityManager
   *   The entity manager service.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entityDisplay
   *   The entity display repository.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, EntityManagerInterface $entityManager, EntityDisplayRepositoryInterface $entityDisplay) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->entityManager = $entityManager;
    $this->entityDisplay = $entityDisplay;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $form['active_description'] = [
      '#type' => 'item',
      '#title' => $this->t("How to use Active button."),
      '#description' => $this->t("Allows to show the block in the content type."),
    ];
    $form['smart_description'] = [
      '#type' => 'item',
      '#title' => $this->t("How to use Mode smart."),
      '#description' => $this->t("Allows the button to take the url if it comes from within the site."),
    ];
    $form['url_description'] = [
      '#type' => 'item',
      '#title' => $this->t("How to use URL Back."),
      '#description' => $this->t("Where do you want the users to go when they click on the button when the smart mode is not activated or they come from outside the site."),
    ];
    foreach ($this->getQuickDisplays() as $content_type) {
      $form[$content_type]['quick_display'] = [
        '#type' => 'checkbox',
        '#title' => $this->t("Active button in ")  . $content_type,
        '#options' => $content_type,
        '#default_value' => $config[$content_type]['quick_display'],
      ];
      $form[$content_type]['smart_mode'] = [
        '#type' => 'checkbox',
        '#title' => $this->t("Active smart mode in ")  . $content_type,
        '#options' => $content_type,
        '#default_value' => $config[$content_type]['smart_mode'],
        '#states' => [
          'visible' => [
            ':input[name$="[' . $content_type . '][quick_display]"]' => [
              'checked' => TRUE,
            ],
           ],
          ],
      ];
      $form[$content_type]['go_back'] = [
        '#type' => 'textfield',
        '#title' => $this->t("URL Back") . " - "  . $content_type,
        //'#description' => $this->t('Where do you want to go back?.'),
        '#default_value' => $config[$content_type]['go_back'],
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    foreach ($this->getQuickDisplays() as $content_type) {
      $values = $form_state->getValues();
      $this->configuration[$content_type]['go_back'] = $values[$content_type]['go_back'];
      $this->configuration[$content_type]['quick_display'] = $values[$content_type]['quick_display'];
      $this->configuration[$content_type]['smart_mode'] = $values[$content_type]['smart_mode'];

    }
  }

  /**
   * {@inheritdoc}
   */

  public function build()
  {
    $config = $this->getConfiguration();
    $node = \Drupal::routeMatch()->getParameter('node');
    if ($node instanceof \Drupal\node\NodeInterface) {
      // You can get nid and anything else you need from the node object.
      $nid = $node->id();
      $entityObj = entity_load('node', $nid);
      $bundle = $node->type->entity->label();
      $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
      if ($config[$bundle]['quick_display'] == '1') {
        if ($config[$bundle]['quick_display'] == '1') {
          if ($config[$bundle]['smart_mode'] == '1') {
            $urlb = $_SERVER['HTTP_REFERER'];
            if (strcmp($urlb, '') !== 0) {
              $urlb = $_SERVER['HTTP_REFERER'];
            } else {
              if (substr($config[$bundle]['go_back'], 0, 1) == '/') {
                $urlb = "/" . $langcode . $config[$bundle]['go_back'];
              } else {
                $urlb = $config[$bundle]['go_back'];
              }
            }
          } else {
            if (substr($config[$bundle]['go_back'], 0, 1) == '/') {
              $urlb = "/" . $langcode . $config[$bundle]['go_back'];
            } else {
              $urlb = $config[$bundle]['go_back'];
            }
          }
        } else {
          $urlb = '#';
        }
        return [
          '#theme' => 'block__go__back',
          '#link' => $urlb,
        ];

      }
    }
  }
  /**
   * Show all display modes of content.
   */
  protected function getQuickDisplays() {
    // Show content types
    $contentTypes = \Drupal::service('entity.manager')->getStorage('node_type')->loadMultiple();

    $contentTypesList = [];
    foreach ($contentTypes as $contentType) {
      $contentTypesList[$contentType->id()] = $contentType->label();
    }
    return $contentTypesList;

  }

  public function getCacheTags() {
    //With this when your node change your block will rebuild
    if ($node = \Drupal::routeMatch()->getParameter('node')) {
      //if there is node add its cachetag
      return Cache::mergeTags(parent::getCacheTags(), array('node:' . $node->id()));
    } else {
      //Return default tags instead.
      return parent::getCacheTags();
    }
  }

  public function getCacheContexts() {
    //if you depends on \Drupal::routeMatch()
    //you must set context of this block with 'route' context tag.
    //Every new route this block will rebuild
    return Cache::mergeContexts(parent::getCacheContexts(), array('route'));
  }

}
